# $Id: 00-compile.t 76 2006-11-29 09:36:01Z miyagawa $

use Test::More tests => 6;

use_ok('XML::Atom');
use_ok('XML::Atom::Entry');
use_ok('XML::Atom::Feed');
#use_ok('XML::Atom::Client');
#use_ok('XML::Atom::Server');
use_ok('XML::Atom::Person');
use_ok('XML::Atom::Content');
use_ok('XML::Atom::Link');
